import threading
import multiprocessing
from queue import Queue
import asyncio
import logging
from datetime import datetime
import time
from typing import Callable, Any, List, Dict


class AsyncBackgroundTask:
    """비동기 백그라운드 작업을 관리하는 클래스"""
    def __init__(self):
        self.tasks = {}
        self.logger = logging.getLogger('AsyncBackgroundTask')
        
    async def add_task(self, task_func: Callable, *args, **kwargs) -> str:
        """비동기 작업 추가"""
        task_id = str(time.time())
        task = asyncio.create_task(self._run_task(task_id, task_func, *args, **kwargs))
        self.tasks[task_id] = task
        return task_id
    
    async def _run_task(self, task_id: str, task_func: Callable, *args, **kwargs):
        """비동기 작업 실행"""
        try:
            start_time = time.time()
            result = await task_func(*args, **kwargs)
            execution_time = time.time() - start_time
            
            self.logger.info(f"Task {task_id} completed in {execution_time:.2f} seconds")
            return {
                'status': 'completed',
                'result': result,
                'error': None,
                'execution_time': execution_time
            }
            
        except Exception as e:
            self.logger.error(f"Task {task_id} failed: {str(e)}")
            return {
                'status': 'failed',
                'result': None,
                'error': str(e),
                'execution_time': time.time() - start_time
            }
    
    async def get_result(self, task_id: str) -> Dict:
        """비동기 작업 결과 조회"""
        if task_id in self.tasks:
            return await self.tasks[task_id]
        return None