
from langchain.storage import LocalFileStore
from langchain.text_splitter import CharacterTextSplitter
from langchain.embeddings import CacheBackedEmbeddings
from langchain_community.document_loaders import PyPDFLoader
from langchain_community.vectorstores import FAISS
from langchain_community.embeddings import OpenAIEmbeddings
from ..ha_constant import HaConstant
import os
import shutil
class FileTask:
    def __init__(self):
        self.video_dir = HaConstant.video_dir
        self.audio_dir = HaConstant.audio_dir
        self.text_dir = HaConstant.text_dir
        self.embed_dir = HaConstant.embed_dir
        self.cache_dir = f"{HaConstant.cache_dir}"
        self.embeddings_dir = f"{HaConstant.cache_embeddings_dir}"
        self.pdf_dir = f"{HaConstant.pdf_dir}"

    def run(self):
        if not os.path.exists(self.video_dir):
            os.makedirs(self.video_dir, exist_ok=True)
        if not os.path.exists(self.pdf_dir):
            os.makedirs(self.pdf_dir, exist_ok=True)
        if not os.path.exists(self.audio_dir):
            os.makedirs(self.audio_dir, exist_ok=True)
        if not os.path.exists(self.text_dir):
            os.makedirs(self.text_dir, exist_ok=True)
        if not os.path.exists(self.embed_dir):
            os.makedirs(self.embed_dir, exist_ok=True)
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir, exist_ok=True)

    def load_file(self, file) -> tuple[str, str]:
        
        file_content = file.read()

        file_path = f"{self.cache_dir}/{file.name}"
        with open(file_path, "wb") as f:
            f.write(file_content)
        return file_path, file_content

    def get_embedding_retriever(self, file):
        file_path = f"{self.cache_dir}/{file.name}"
        cache_dir = LocalFileStore(f"{self.embeddings_dir}/{file.name}")
        splitter = CharacterTextSplitter.from_tiktoken_encoder(
            separator="\n",
            chunk_size=600,
            chunk_overlap=100,
        )
        loader = PyPDFLoader(file_path)
        docs = loader.load_and_split(text_splitter=splitter)
        embeddings = OpenAIEmbeddings()
        cached_embeddings = CacheBackedEmbeddings.from_bytes_store(embeddings, cache_dir)
        vectorstore = FAISS.from_documents(docs, cached_embeddings)
        retriever = vectorstore.as_retriever()
        return retriever
    

    def exist_audio_dir(self) -> bool:
        return os.path.exists(self.audio_dir)
    
    def create_audio_dir(self,):
        os.makedirs(f"{self.audio_dir}", exist_ok=True)

    def exist_audio_file(self, audio_id:str) -> bool:
        file_path = f"{self.audio_dir}/{audio_id}.mp3"
        return os.path.exists(file_path)
    
    def exist_text_file(self, audio_id:str) -> bool:
        file_path = f"{self.embed_dir}/{audio_id}.txt"
        return os.path.exists(file_path)
    
    def exist_embed_file(self, audio_id:str) -> bool:
        file_path = f"{self.embed_dir}/{audio_id}"
        return os.path.exists(file_path)
    
    def delete_audio_file(self, audio_id:str):
        file_path = f"{self.audio_dir}/{audio_id}.mp3"
        if os.path.exists(file_path):
            os.remove(file_path)

    def delete_embed_file(self, audio_id:str):
        file_path = f"{self.embed_dir}/{audio_id}"
        if os.path.exists(file_path):
            os.remove(file_path)

    def delete_text_file(self, audio_id:str) -> bool:
        file_path = f"{self.embed_dir}/{audio_id}.txt"
        if os.path.exists(file_path):
            os.remove(file_path)        
    


    def delete_folder_contents(self, folder_path):
        """
        폴더 내의 모든 파일과 하위 폴더를 삭제하는 함수
        
        Parameters:
        folder_path (str): 삭제할 폴더 경로
        
        Returns:
        bool: 삭제 성공 여부
        """
        try:
            for item in os.listdir(folder_path):
                item_path = os.path.join(folder_path, item)
                if os.path.isfile(item_path):
                    os.remove(item_path)
                    print(f"파일 삭제 완료: {item}")
                elif os.path.isdir(item_path):
                    shutil.rmtree(item_path)
                    print(f"하위 폴더 삭제 완료: {item}")
            return True
            
        except Exception as e:
            print(f'폴더 내용 삭제 중 오류 발생: {str(e)}')
            return False

    def delete_folder(self, folder_path):
        """
        폴더와 그 내용을 모두 삭제하는 함수
        
        Parameters:
        folder_path (str): 삭제할 폴더 경로
        
        Returns:
        bool: 삭제 성공 여부
        """
        try:
            if os.path.exists(folder_path):
                shutil.rmtree(folder_path)
                print(f"폴더 삭제 완료: {folder_path}")
                return True
            else:
                print(f"폴더가 존재하지 않습니다: {folder_path}")
                return False
                
        except Exception as e:
            print(f'폴더 삭제 중 오류 발생: {str(e)}')
            return False

    def delete_folder_contents_and_keep_folder(self, folder_path):
        """
        폴더는 유지하고 내용물만 삭제하는 함수
        
        Parameters:
        folder_path (str): 내용을 삭제할 폴더 경로
        
        Returns:
        bool: 삭제 성공 여부
        """
        try:
            if not os.path.exists(folder_path):
                print(f"폴더가 존재하지 않습니다: {folder_path}")
                return False
                
            # 폴더 내용 삭제
            self.delete_folder_contents(folder_path)
            print(f"폴더 내용 삭제 완료 (폴더 유지): {folder_path}")
            return True
            
        except Exception as e:
            print(f'폴더 내용 삭제 중 오류 발생: {str(e)}')
            return False    