
import os
from langchain_community.document_loaders import TextLoader
from langchain_openai.embeddings import OpenAIEmbeddings
from langchain.embeddings import CacheBackedEmbeddings
from langchain_community.vectorstores.faiss import FAISS
from langchain.storage import LocalFileStore
from langchain.text_splitter import CharacterTextSplitter
from langchain_openai import ChatOpenAI
from langchain_core.output_parsers import StrOutputParser
from ..tasks.ha_common_task import HaCommonTask
from ..ha_constant import HaConstant
from langchain.chains import create_retrieval_chain
from langchain.chains.combine_documents import create_stuff_documents_chain
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.output_parsers import PydanticOutputParser
from dotenv import load_dotenv
from pydantic import BaseModel, Field
from typing import List

load_dotenv()

class ReportOnePage(BaseModel):
    title: str = Field(description="제목")
    summary: str = Field(description="세줄 요약")   
    problem: str = Field(description="문제")
    answer: str = Field(description="추천답안")

class BookDetail(BaseModel):    
    title: str = Field(description="책 제목")
    link: str = Field(description="교보문고, 알라딘, 예스24 링크")
    image: str = Field(description="썸네일 url")
    author: str = Field(description="저자")
    discount: str = Field(description="할인 가격")
    pubdate: str = Field(description="출판일")
    isbn: str = Field(description="ISBN")
    description: str = Field(description="책 설명")


class BookList(BaseModel):
    result: List[str] = None

class SocialDetail(BaseModel):
    instagram: str = Field(description="인스타그램 링크")
    twitter: str = Field(description="트위터 링크")
    facebook: str = Field(description="페이스북 링크")

class SocialList(BaseModel):
    result: List[str] = None

class HumanDetail(BaseModel):
    name: str = Field(description="사람 이름")
    link: str = Field(description="인스타그램, 트위터, 페이스북 링크")
class HumanNameList(BaseModel):
    result: List[HumanDetail] = None

class TitleList(BaseModel):
    result: List[str] = None

class HashTagList(BaseModel):
    result: List[str] = None


class AnswerList(BaseModel):
    a : str = None
    b : str = None
    c : str = None
    d : str = None

class QuestionList(BaseModel):
    question: str = None
    answer: AnswerList = None

class QuestionForm(BaseModel):
    result: List[QuestionList] = None

class RAGTask:
    def __init__(self):
        self.common_task = HaCommonTask()
        self.openai_api_key = os.getenv("OPENAI_API_KEY")

    def get_basic_chain(self, audio_id:str):
        openai_api_key = self.openai_api_key
 
        cache_dir = LocalFileStore(f"{HaConstant.embed_dir}/{audio_id}")
        splitter = CharacterTextSplitter.from_tiktoken_encoder(
            separator="\n",
            chunk_size=600,
            chunk_overlap=100,
        )
        
        loader = TextLoader(HaConstant.embed_dir, encoding="utf-8")
        docs = loader.load_and_split(text_splitter=splitter)
        embeddings = OpenAIEmbeddings(api_key=openai_api_key)
        # print(embeddings)
        cached_embeddings = CacheBackedEmbeddings.from_bytes_store(embeddings, cache_dir)

        vectorstore = FAISS.from_documents(docs, cached_embeddings)
        retriever = vectorstore.as_retriever()
        llm = ChatOpenAI(model="gpt-4o-mini", api_key=openai_api_key)

        system_prompt = ("""
            Use the given context to answer the question. 
            If you don't know the answer, say you don't know. 
            Use three sentence maximum and keep the answer concise. 
            모든 대답은 한글로 해줘
            Context: {context}
            FORMAT:
            {format_instructions}
        """
        )

        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", system_prompt),
                ("human", "{input}"),
            ]
        )

        prompt = prompt.partial(format_instructions=parser.get_format_instructions())

        parser = PydanticOutputParser(pydantic_object=ReportOnePage)
        question_answer_chain = create_stuff_documents_chain(llm, prompt)
        chain = create_retrieval_chain(retriever, question_answer_chain)
       

        return chain


    def get_chain(self, url:str, file_path:str, index:int):
        openai_api_key = self.openai_api_key  
        # print(f"openai_api_key: {openai_api_key}")
        audio_id = self.common_task.get_audio_id(url)
 
        cache_dir = LocalFileStore(f"{HaConstant.embed_dir}/{audio_id}")
        splitter = CharacterTextSplitter.from_tiktoken_encoder(
            separator="\n",
            chunk_size=600,
            chunk_overlap=100,
        )
        print(f"file_path: {file_path}")
        loader = TextLoader(file_path, encoding="utf-8")
        docs = loader.load_and_split(text_splitter=splitter)
        embeddings = OpenAIEmbeddings(api_key=openai_api_key)
        # print(embeddings)
        cached_embeddings = CacheBackedEmbeddings.from_bytes_store(embeddings, cache_dir)

        vectorstore = FAISS.from_documents(docs, cached_embeddings)
        retriever = vectorstore.as_retriever()
        llm = ChatOpenAI(model="gpt-4o-mini", api_key=openai_api_key)
        
        parser = None
        if index == 0:
            parser = PydanticOutputParser(pydantic_object=ReportOnePage)
        elif index == 1:
            parser = PydanticOutputParser(pydantic_object=BookList)
        elif index == 2:
            parser = PydanticOutputParser(pydantic_object=HumanNameList)
        elif index == 3:
            parser = PydanticOutputParser(pydantic_object=TitleList)    
        elif index == 4:
            parser = PydanticOutputParser(pydantic_object=HashTagList)
        elif index == 5:
            parser = PydanticOutputParser(pydantic_object=QuestionForm)
        else:
            parser = PydanticOutputParser(pydantic_object=ReportOnePage)
        system_prompt = ("""
            Use the given context to answer the question. 
            If you don't know the answer, say you don't know. 
            Use three sentence maximum and keep the answer concise. 
            모든 대답은 한글로 해줘
            Context: {context}
            FORMAT:
            {format_instructions}
        """
        )

        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", system_prompt),
                ("human", "{input}"),
            ]
        )

        prompt = prompt.partial(format_instructions=parser.get_format_instructions())

    
        question_answer_chain = create_stuff_documents_chain(llm, prompt)
        chain = create_retrieval_chain(retriever, question_answer_chain)
       

        return chain